%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
clear all
close all

%% initialize environment to run smt
addpath ('..');
smt_setup;

%% Open faces database
load ('./data/faces_orl_644_28_23.mat')
[p,n] = size(Y);

DisplayImageVectors2(Y, 28,23);


%% Compute Fast SMT Transform
r=4.0;
eigenvalue_floor = 0.5; %set a floor on the eigenvalue estimates. See Guangzhi et al. on IEEE transactions on Image processing, March, 2011 for further discussion
[mask_indices, neighbor_map] = SMT_find_neighborhood_from_image_mask(ones(28,23));
[rotation_list, lambda_est, K_max, avg_smt_logLikelihood] = SMT_build_neighborhood_model_crossvalidation(Y, 10, eigenvalue_floor, r, 8, 28, 23);
[sorted_vals, sorted_indices] = sort (lambda_est,'descend');
I = eye(p,p);

E = SMT_multiply(rotation_list, I);
DisplayImageVectors2(E(:,sorted_indices(160:-1:1)), 28,23);


%% Compute PCA eigenfaces
[U, S, V] = svd (Y,0);
DisplayImageVectors2(U(:, 40:-1:1), 28, 23)


%% Cross-validated likelihood
ll0 = 0;
all_indices = (1:40)';
for i=1:10
   all_indices = circshift (all_indices, 4);
   train_indices =  all_indices (1:36);
   test_indices = all_indices (37:40);
   Y_train = Y(:, train_indices);
   Y_test  = Y(:, test_indices);
   [U, S, V] = svd (Y_train,0);
   
   x = sum((U' * Y_test).^2,2);
   
   ll = ll0 + SMT_compute_likelihood(x, diag(S), 4);
end
avg_eigen_logLikelihood = (ll/10)*ones(1, length (avg_smt_logLikelihood));


%% Plot comparing likelihoods of both methods
figure;
set (0,'defaultaxesfontsize',14);
r_range = r *(1:length(avg_smt_logLikelihood))./length(avg_smt_logLikelihood);

plot(r_range, avg_smt_logLikelihood/4,'Linewidth',2);
hold on
plot(r_range, avg_eigen_logLikelihood, 'r','Linewidth',2);
xlabel ('r (SMT rotations/pixel)');
ylabel ('log likelihood');
grid on
title ('SMT vs. PCA cross-validated log-likelihood');
legend('SMT LL','PCA LL')