%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [rotation_list, lambda, PointedBySizes, PointedBySizesOfSelectedPairs, smt_likelihood] = AnalyzeFastSMT (Y_train, K, neighbor_list_max_len, image_height, image_width, Y_test)       
    
    if nargin < 6
        likelihood_flag = 0;
    else
        likelihood_flag = 1;
    end
    
    
    %get matrix dimensions
    [P1, train_set_size] = size (Y_train);
    
    smt_train = FastSMT4 (Y_train);
    
    if (image_height > 0 && image_width > 0)
        smt_train.addImageNeighbors(image_height, image_width, 1);
    else
        smt_train.addAllNeighbors();
    end
    
    if (neighbor_list_max_len > 0)
        smt_train.pruneNeighborhoodList(neighbor_list_max_len);
    end
    
    if likelihood_flag == 1
        %allocate memory for the likelihood values at each rotation
        smt_likelihood = zeros(K,1);
        
        [P2, test_set_size] = size(Y_test);
        
        if (P1 ~= P2)
            disp('ERROR: Y_train and Y_test dimensions DO NOT match!')
            return;
        end
        smt_test = FastSMT4(Y_test);
        
        %run SMT step by step and compute the likelihood of Y_test at the
        %end of each step
        for k = 1:K
            smt_train.PerformRotations(1);
            rotation = smt_train.MatlabGetLastRotations(1);
            
            smt_test.MatlabApplyRotations(rotation);
            
            train_eigenvalues = smt_train.getDiagonal();
            test_eigenvalues = smt_test.getDiagonal();
            smt_likelihood(k) = (SMT_compute_likelihood(test_eigenvalues, train_eigenvalues, test_set_size))/test_set_size;% modified to calculate the average
            
            if (k == 1)
                PointedBySizes = zeros (K, p);
            end
            PointedBySizes(k,:) = smt_train.getPointedByNumbers();
         end
    else
        PointedBySizesOfSelectedPairs = zeros (1, K);
        for k = 1:K
            smt_train.PerformRotations(1);
            rotation = smt_train.MatlabGetLastRotations(1);
           
            
      
            PointedBy = smt_train.getPointedByNumbers();
            ik = round(rotation(1));
            jk = round(rotation(2));
            PointedBySizesOfSelectedPairs(k) = PointedBy(ik) + PointedBy(jk);
        end
         
        smt_likelihood = -1;
        % if we're not computing likelihoods, run all K steps at once!
        %smt_train.PerformRotations(K);
        PointedBySizes = smt_train.getPointedByNumbers();
    end
    
    %rotation_list = smt_train.MatlabGetLastRotations(-1);
    rotation_list = smt_train.MatlabGetRotations();
    
    lambda = smt_train.getDiagonal();
end
