%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [ handle ] = DisplayImageVectors2(vectorset, height, width, outputepsfile)

[p,n]= size(vectorset);
columns = ceil(n/10);
rows = 10;
canvas = zeros (rows*height, columns*width);

figure
for index=1:n
    vector = vectorset(:,index);
    
    %rescale pixel intensities
    rescaled_vector = round (255*(vector - min(vector))./(max(vector)-min(vector))); 
    
    %reshape vector to form image
    im = reshape (rescaled_vector, height, width);
    %%m2 = round(255*(im + min(im))./ (max(im) - min(im)));
    
    %draw the image
    index2 = n - index + 1;

    ndx = mod (index2-1, columns);
    ndy = floor ((index2-1)/ columns);
    canvas (ndy*height+(1:height), ndx*width+(1:width)) = im;
end
%adjust the colormap
handle = image(canvas);
set (gca, 'Position',[0, 0, 1, 1]);
set (gcf, 'Units','pixels');
set (gcf, 'Position',[0, 0, columns*width, rows*height]);

axis off
c=(0:255)/255;
cm=[c' c' c'];
colormap(cm);

if (nargin > 3)
    set (gcf, 'color','white');
    print (gcf, '-deps', outputepsfile);
end
