%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [rotation_list, lambda_est, K, avg_likelihood] = SMT_build_model_crossvalidation(dataset, crossvalid_kfold, r, version, M, height, width)

    if (strcmp (version, 'FastSMT'))
        [K_max, avg_logLikelihood] = SMT_crossvalidation (dataset, crossvalid_kfold, r, version, M, height, width);
        [rotation_list, lambda_est] = SMT_build_neighborhood_model2 (dataset, K_max, M, height, width);
    elseif (strcmp (version,'FullSMT'))
        [K_max, avg_logLikelihood] = SMT_crossvalidation (dataset, crossvalid_kfold, r, version, M, height, width);
    end
        
%% choose the number of steps K by crossvalidation
    [P,N] = size (dataset);
    nsamples = crossvalid_kfold * floor(N / crossvalid_kfold);
    [Likelihoods] = SMT_crossvalidation(dataset(:,1:nsamples), crossvalid_kfold, r, M, height, width);
    
    avg_likelihood = mean (Likelihoods,2);
    [maximizer, K] = max (avg_likelihood); %select K that maximizes the expected log likelihood
    
    %% Estimate the SMT Model 
    [rotation_list, lambda_est] = SMT_build_neighborhood_model2 (dataset, K, M, height, width);
end