%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [E,Lambda]=SMT_compute_looc(Y,alpha, debug_flag)
%%% use diag(S) as the shrinkage estimate  %%%%
%%% choose alpha to maximize LOOL %%%%%

[N,M]=size(Y);
S=Y*Y'/M;

if nargin == 1
    alpha = 0.1:0.1:1;
end
if nargin < 3
    debug_flag = 0;
end

avg_LooL  = zeros (1,length(alpha));
avg_LooL1 = zeros (1,length(alpha));

%%%%  the simplifed version for LOOL with the biased estimator %%
for i=1:size(alpha,2)
    disp (['  - SMT_compute_looc iteration ',mat2str(i),' of ',mat2str(size(alpha,2))]);
    alpha1=alpha(i);
    alpha2=1-alpha1;
    
    G=alpha1*diag(diag(S))+alpha2*(M)/(M-1)*S;
    k=alpha2/(M-1);
    [E,Lambda]=eig(G);
    Ginv=E*diag((1./diag(Lambda)))*E';
    logGdet=sum(log(diag(Lambda)));
    log_likelihood=zeros(M,1);
    log_likelihood1 = zeros (M,1);
    %disp ('   -- Starting iterations over samples');
    for m=1:M
        %disp (['    ---iteration ',mat2str(m)])
        d=Y(:,m)'*Ginv*Y(:,m);
        log_likelihood(m)=-M/2*log(2*pi)-1/2*logGdet-1/2*log(1-k*d)-1/2*(d/(1-k*d));
        log_likelihood1(m)=-(N/2)*log(2*pi)-(1/2)*logGdet-(1/2)*d;
    end
    avg_LooL(i)=sum(log_likelihood)/M;
    avg_LooL1(i) = sum (log_likelihood1)/M;
end

[val_max,i_max]=max(avg_LooL);
alpha_opt=alpha(i_max);

if (debug_flag == 1)
    figure
    subplot (1,2,1); plot (alpha, avg_LooL);
    subplot (1,2,2); plot (alpha, avg_LooL1);
end


R=alpha_opt*diag(diag(S))+(1-alpha_opt)*S;
[E,Lambda]=eig(R);
[Lambda,I] = sort(diag(Lambda),'descend');
E=E(:,I);
Lambda=diag(Lambda);
