%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [K_max, avg_logLikelihood]=SMT_crossvalidation(Y, kfold, eigenvalues_floor, r, version, neighborhood_max_len, image_height, image_width)
%% get parameters of the dataset
[P,M] = size (Y);


delta= floor(M/kfold);

m_used = kfold*delta;

K=round (r*P);

Likelihoods = zeros (K,kfold);

if (nargin < 7)
    image_width = -1;
end
if (nargin < 6)
    image_height = -1;
end
if (nargin < 5)
    neighborhood_max_len = -1;
end


%% Cross-validation iterations
for k=1:kfold
    fullrange = circshift ((1:m_used)', k*delta);
    test_range = fullrange(1:delta)';
    train_range = fullrange(delta+1:m_used)';
    
    Y_train = Y(:, train_range);
    Y_test = Y(:, test_range);
    
    [P,M_train] = size (Y_train);
    [P,M_test] = size (Y_test); 
    
    if (strcmp(version,'FastSMT'))
        [Rotations, lambda, likelihood] = SMT_build_neighborhood_model2 (Y_train, eigenvalues_floor, K, neighborhood_max_len, image_height, image_width, Y_test);
    elseif (strcmp(version,'FullSMT'))
        S = (1/M_train)*Y_train*Y_train';
        [E_smt, Lambda_smt, SMTArray, likelihood] = SMT_build_model (S, K, Y_test);
    end
    Likelihoods(:,k) = likelihood;
end

avg_logLikelihood = mean (Likelihoods, 2);
[avg_logLikelihood_max, K_max] = max (avg_logLikelihood);

end
