%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [mask_indices, neighbor_map] = SMT_find_neighborhood_from_image_mask (image_mask)
%image_mask = feret_mask;
image_mask_complement = (double(image_mask) - 1);
[image_height, image_width] = size(image_mask);
p = image_height*image_width;

vector_mask = reshape (image_mask, p,1);
valid_indices = cumsum (double(vector_mask));

vector_indices = round(double (vector_mask) .* valid_indices);
mask_with_indices = reshape (vector_indices, image_height, image_width) + image_mask_complement;

mask_indices = find (vector_mask ~= 0);
new_p = length (mask_indices);
neighbor_map = zeros (new_p, 8);

% ignore corner points because they are very likely outside the mask area
if (mask_with_indices(1,1) ~= -1)
    neighbor_map(mask_with_indices(1,1), 1) = -1;
    neighbor_map(mask_with_indices(1,1), 2) = -1;
    neighbor_map(mask_with_indices(1,1), 3) = -1;
    neighbor_map(mask_with_indices(1,1), 4) = -1;
    neighbor_map(mask_with_indices(1,1), 5) = mask_with_indices(1, 2);
    neighbor_map(mask_with_indices(1,1), 6) = -1;
    neighbor_map(mask_with_indices(1,1), 7) = mask_with_indices(2,1);
    neighbor_map(mask_with_indices(1,1), 8) = mask_with_indices(2,2);
end

if (mask_with_indices(1,image_width) ~= -1)
    neighbor_map(mask_with_indices(1,image_width), 1) = -1;%mask_with_indices(1-1, image_width-1);
    neighbor_map(mask_with_indices(1,image_width), 2) = -1;%mask_with_indices(1-1, image_width);
    neighbor_map(mask_with_indices(1,image_width), 3) = -1;%mask_with_indices(1-1, image_width+1);
    neighbor_map(mask_with_indices(1,image_width), 4) = mask_with_indices(1, image_width-1);
    neighbor_map(mask_with_indices(1,image_width), 5) = -1; %mask_with_indices(1, image_width+1);
    neighbor_map(mask_with_indices(1,image_width), 6) = mask_with_indices(1+1, image_width-1);
    neighbor_map(mask_with_indices(1,image_width), 7) = mask_with_indices(1+1,image_width);
    neighbor_map(mask_with_indices(1,image_width), 8) = -1;%mask_with_indices(1+1,image_width+1);
end

if (mask_with_indices(image_height,1) ~= -1)
    neighbor_map(mask_with_indices(image_height,1), 1) = -1; %mask_with_indices(image_height-1, 1-1);
    neighbor_map(mask_with_indices(image_height,1), 2) = mask_with_indices(image_height-1, 1);
    neighbor_map(mask_with_indices(image_height,1), 3) = mask_with_indices(image_height-1, 1+1);
    neighbor_map(mask_with_indices(image_height,1), 4) = -1; %mask_with_indices(image_height, 1-1);
    neighbor_map(mask_with_indices(image_height,1), 5) = mask_with_indices(image_height, 1+1);
    neighbor_map(mask_with_indices(image_height,1), 6) = -1; %mask_with_indices(image_height+1, 1-1);
    neighbor_map(mask_with_indices(image_height,1), 7) = -1; %mask_with_indices(image_height+1,1);
    neighbor_map(mask_with_indices(image_height,1), 8) = -1; %mask_with_indices(image_height+1,1+1);
end

if (mask_with_indices(image_height,image_width) ~= -1)
    neighbor_map(mask_with_indices(image_height,image_width), 1) = mask_with_indices(image_height-1, image_width-1);
    neighbor_map(mask_with_indices(image_height,image_width), 2) = mask_with_indices(image_height-1, image_width);
    neighbor_map(mask_with_indices(image_height,image_width), 3) = -1;%mask_with_indices(image_height-1, image_width+1);
    neighbor_map(mask_with_indices(image_height,image_width), 4) = mask_with_indices(image_height, image_width-1);
    neighbor_map(mask_with_indices(image_height,image_width), 5) = -1;%mask_with_indices(image_height, image_width+1);
    neighbor_map(mask_with_indices(image_height,image_width), 6) = -1;%mask_with_indices(image_height+1, image_width-1);
    neighbor_map(mask_with_indices(image_height,image_width), 7) = -1;%mask_with_indices(image_height+1,image_width);
    neighbor_map(mask_with_indices(image_height,image_width), 8) = -1;%mask_with_indices(image_height+1,image_width+1);
end

for i=2:image_height-1
    if (mask_with_indices(i,1) ~= -1)
        neighbor_map(mask_with_indices(i,1), 1) = -1; %mask_with_indices(i-1, 1-1);
            neighbor_map(mask_with_indices(i,1), 2) = mask_with_indices(i-1, 1);
            neighbor_map(mask_with_indices(i,1), 3) = mask_with_indices(i-1, 1+1);
            neighbor_map(mask_with_indices(i,1), 4) = -1; %mask_with_indices(i, 1-1);
            neighbor_map(mask_with_indices(i,1), 5) = mask_with_indices(i, 1+1);
            neighbor_map(mask_with_indices(i,1), 6) = -1; %mask_with_indices(i+1, 1-1);
            neighbor_map(mask_with_indices(i,1), 7) = mask_with_indices(i+1,1);
            neighbor_map(mask_with_indices(i,1), 8) = mask_with_indices(i+1,1+1);
    end
    if (mask_with_indices(i, image_width) ~= -1)
        neighbor_map(mask_with_indices(i,image_width), 1) = -1; %mask_with_indices(i-1, image_width-1);
            neighbor_map(mask_with_indices(i,image_width), 2) = mask_with_indices(i-1, image_width);
            neighbor_map(mask_with_indices(i,image_width), 3) = mask_with_indices(i-1, image_width-1);
            neighbor_map(mask_with_indices(i,image_width), 4) = mask_with_indices(i, image_width-1);
            neighbor_map(mask_with_indices(i,image_width), 5) = -1; %mask_with_indices(i, image_width+1);
            neighbor_map(mask_with_indices(i,image_width), 6) = mask_with_indices(i+1, image_width-1);
            neighbor_map(mask_with_indices(i,image_width), 7) = mask_with_indices(i+1,image_width);
            neighbor_map(mask_with_indices(i,image_width), 8) = -1; %mask_with_indices(i+1,image_width+1);
    end
end
for j=2:image_width-1
    if (mask_with_indices(1,j) ~= -1)
            neighbor_map(mask_with_indices(1,j), 1) = -1; %mask_with_indices(1, j-1);
            neighbor_map(mask_with_indices(1,j), 2) = -1; %mask_with_indices(1, j+1);
            neighbor_map(mask_with_indices(1,j), 3) = -1; %mask_with_indices(2, j-1);
            neighbor_map(mask_with_indices(1,j), 4) = mask_with_indices(1, j-1);
            neighbor_map(mask_with_indices(1,j), 5) = mask_with_indices(1, j+1);
            neighbor_map(mask_with_indices(1,j), 6) = mask_with_indices(2, j-1);
            neighbor_map(mask_with_indices(1,j), 7) = mask_with_indices(2,j);
            neighbor_map(mask_with_indices(1,j), 8) = mask_with_indices(2,j+1);
    end
        if (mask_with_indices(image_height,j) ~= -1)
            neighbor_map(mask_with_indices(image_height,j), 1) = -1; %mask_with_indices(image_height, j-1);
            neighbor_map(mask_with_indices(image_height,j), 2) = -1; %mask_with_indices(image_height, j+1);
            neighbor_map(mask_with_indices(image_height,j), 3) = -1; %mask_with_indices(image_height+1, j-1);
            neighbor_map(mask_with_indices(image_height,j), 4) = mask_with_indices(image_height, j-1);
            neighbor_map(mask_with_indices(image_height,j), 5) = mask_with_indices(image_height, j+1);
            neighbor_map(mask_with_indices(image_height,j), 6) = mask_with_indices(image_height-1, j-1);
            neighbor_map(mask_with_indices(image_height,j), 7) = mask_with_indices(image_height-1,j);
            neighbor_map(mask_with_indices(image_height,j), 8) = mask_with_indices(image_height-1,j+1);
        end
end
for i = 2:image_height-1
    for j = 2:image_width-1
        if (mask_with_indices(i,j) ~= -1)
            neighbor_map(mask_with_indices(i,j), 1) = mask_with_indices(i-1, j-1);
            neighbor_map(mask_with_indices(i,j), 2) = mask_with_indices(i-1, j);
            neighbor_map(mask_with_indices(i,j), 3) = mask_with_indices(i-1, j+1);
            neighbor_map(mask_with_indices(i,j), 4) = mask_with_indices(i, j-1);
            neighbor_map(mask_with_indices(i,j), 5) = mask_with_indices(i, j+1);
            neighbor_map(mask_with_indices(i,j), 6) = mask_with_indices(i+1, j-1);
            neighbor_map(mask_with_indices(i,j), 7) = mask_with_indices(i+1,j);
            neighbor_map(mask_with_indices(i,j), 8) = mask_with_indices(i+1,j+1);
        end
    end
end
