%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [R, E, lambda_est] = SMT_neighborhood_covariance_estimation (Y, r, crossvalid_flag, kfold, M, image_height, image_width)
% [R, E, lambda] = SMT_full_covariance_estimation (Y, r, crossvalid_flag, kfold)
% 
% Y: p x n matrix containing n p-dimensional samples
% r: perfrom r*p rotations or search for optimal num. of rotations in the
%    0..r*p using crossvalidation
% crossvalid_flag: 'crossvalidation' or ''
% kfold: split Y in kfold ways for crossvalidation 

    [P,N] = size (Y);
    S = (1/N)*Y*Y';
  
    if strcmp (crossvalid_flag, 'crossvalidation')
        [rotation_list, lambda] = SMT_build_neighborhood_model_crossvalidation(Y, kfold, r, M, image_height, image_width);
    else
        K = round(P*r);
        [rotation_list,lambda] = SMT_build_neighborhood_model2(Y, K, M, image_height, image_width);
    end
    
    [lambda_est, indices] = sort (lambda, 'descend');
    
    I = eye (P,P);
    I_sorted = I(:,indices);
    
    E = SMT_multiply(rotation_list, I_sorted);
    R = E*diag(lambda_est)*E';
end