%%%
% Copyright (c) 2010, Leonardo R. Bachega and Charles A. Bouman
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the distribution.
%     * Neither the name of the Purdue University nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL Leonardo R. Bachega and Charles A. Bouman BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%
function [listTags, vPruneMap] = SMT_prune_multiplications (SMTArray, vPruneMap)
    
    % listTags contains the tags for each rotation:
    % 0 - rotation is not necessary
    % 1 - only update i-th coordinate
    % 2 - only update j-th coordinate
    % 3 - perform "full" rotation
    
    K = size (SMTArray, 1);
    listTags = 3 * ones(K,1);
    for k=K:-1:1
        
        i = SMTArray(k,1);
        j = SMTArray(k,2);
        
        if (vPruneMap(i) == 0 && vPruneMap(j) == 0)
            listTags(k) = 0; %%this rotation isn't necessary
        elseif (vPruneMap(i) == 0 && vPruneMap(j) == 1)
            listTags(k) = 2; %%only multiply j-th coordinate
            vPruneMap(i) = 1; 
        elseif (vPruneMap(i) == 1 && vPruneMap(j) == 0) 
            listTags(k) = 1; %%only multiply i-th coordinate
            vPruneMap(j) = 1;
        end
    end
end